/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <string>
#include <sstream>
#include <iomanip>

#include <errno.h>
#include <cstring>
#ifdef WIN32
#include <process.h>
#endif
#include <glibmm/i18n.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/box.h>
#include <gtkmm/main.h>
#include <glibmm/ustring.h>
#include "AirportInfoDialog.h"
#include "Airport.h"
#include "Route.h"
#include "Util.h"

AirportInfoDialog::AirportInfoDialog(std::string *GladeFilename, DataStore *Data, Config *Cfg, Airport *AirportData, Route *RouteData)
{
	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "AirportInfoDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog(_("Error reading data file: ")+*GladeFilename+"\n"+_("Program cannot continue."),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("AirportInfoDialog",GtkDialog);
	refXml->get_widget("AddtoRouteButton",AddtoRouteButton);
	refXml->get_widget("TitleLabel",TitleLabel);
	refXml->get_widget("ElevMagVarLabel",ElevMagVarLabel);
	refXml->get_widget("LocFmt1Label",LocFmt1Label);
	refXml->get_widget("LocFmt2Label",LocFmt2Label);
	refXml->get_widget("LocFmt3Label",LocFmt3Label);
	refXml->get_widget("DayNightFrame",DayNightFrame);
	refXml->get_widget("MornTwiLabel",MornTwiLabel);
	refXml->get_widget("SunriseLabel",SunriseLabel);
	refXml->get_widget("SunsetLabel",SunsetLabel);
	refXml->get_widget("EveTwiLabel",EveTwiLabel);
	refXml->get_widget("RunwaysFrame",RunwaysFrame);
	refXml->get_widget("RunwaysAlignment",RunwaysAlignment);
	refXml->get_widget("ApproachPlatesFrame",ApproachPlatesFrame);
	refXml->get_widget("ApproachPlatesScrolledWindow",ApproachPlatesScrolledWindow);
	refXml->get_widget("METARFrame",METARFrame);
	refXml->get_widget("METARTextView",METARTextView);
	refXml->get_widget("CloseButton",CloseButton);

	AddtoRouteButton->signal_clicked().connect(sigc::mem_fun(this,&AirportInfoDialog::on_AddtoRouteButton_clicked));
	CloseButton->signal_clicked().connect(sigc::mem_fun(this,&AirportInfoDialog::on_CloseButton_clicked));

	this->Cfg=Cfg;
	this->Data=Data;
	this->AirportData=AirportData;
	this->RouteData=RouteData;

	if (!RouteData->GetFilename().length()) {
		AddtoRouteButton->set_sensitive(false);
	}

	TitleLabel->set_markup(Glib::ustring("<b>")+AirportData->GetName()+"</b> ("+AirportData->GetID()+")");

	ElevMagVarLabel->set_text(
		_("Elevation")+
		Glib::ustring(": ")+
		Glib::ustring::format(AirportData->GetElevation())+
		" ft, "+
		_("MagVar")+
		": "+
		Glib::ustring::format(AirportData->GetMagVar())+
		"°"
		);

	std::string LocStr;
	Util::LonLatToDMMSSPSSSS(AirportData->GetLon(),AirportData->GetLat(),&LocStr);
	LocFmt1Label->set_text(LocStr);

	Util::LonLatToDMMPMMMMMM(AirportData->GetLon(),AirportData->GetLat(),&LocStr);
	LocFmt2Label->set_text(LocStr);

	Util::LonLatToDPDDDDDDD(AirportData->GetLon(),AirportData->GetLat(),&LocStr);
	LocFmt3Label->set_text(LocStr);

#ifdef HAVE_LIBNOVA
	struct tm bdtime;

	Util::GmTime_r(&bdtime,AirportData->GetMornCivilTwilight());
	MornTwiLabel->set_text(Util::BDTimeToMM_DD_HH_MM(&bdtime)+"Z");

	Util::GmTime_r(&bdtime,AirportData->GetSunrise());
	SunriseLabel->set_text(Util::BDTimeToMM_DD_HH_MM(&bdtime)+"Z");

	Util::GmTime_r(&bdtime,AirportData->GetSunset());
	SunsetLabel->set_text(Util::BDTimeToMM_DD_HH_MM(&bdtime)+"Z");

	Util::GmTime_r(&bdtime,AirportData->GetEveCivilTwilight());
	EveTwiLabel->set_text(Util::BDTimeToMM_DD_HH_MM(&bdtime)+"Z");
#else
	DayNightFrame->hide();
#endif

	if (AirportData->GetRunways()->size()) {
		Gtk::VBox *RunwayVBox=new Gtk::VBox();

		for (std::vector<Runway*>::iterator i=AirportData->GetRunways()->begin(); i!=AirportData->GetRunways()->end();i++) {
			Gtk::Label *DataLabel=new Gtk::Label();

			DataLabel->set_alignment(0.0,0.5);
			std::string RunwayData;
			RunwayData=
				(*i)->GetID()+": "+
				Glib::ustring::format((*i)->GetLength())+
				"' x "+
				Glib::ustring::format((*i)->GetWidth())+
				"' ";
			if ((*i)->GetDisplaced())
				RunwayData+="("+Glib::ustring::format((*i)->GetDisplaced())+"' displaced); ";
			RunwayData+=
				"("+
				Glib::ustring::format((*i)->GetLat())+
				", "+
				Glib::ustring::format((*i)->GetLon())+
				") TDZE="+
				Glib::ustring::format((*i)->GetElevation())+
				"', Mag Brg="+
				Glib::ustring::format((*i)->GetMagBearing())+
				"°";
			DataLabel->set_text(RunwayData);
			RunwayVBox->add(*DataLabel);
		}

		RunwaysAlignment->add(*RunwayVBox);
		RunwaysAlignment->show_all();
	} else {
		RunwaysFrame->hide();
	}

	if (AirportData->GetApproachPlates()->size()) {
		Gtk::VBox *ApproachPlatesVBox=new Gtk::VBox;

		for (std::vector<ApproachPlate*>::iterator i=AirportData->GetApproachPlates()->begin(); i!=AirportData->GetApproachPlates()->end();i++) {
			Gtk::Button *PlateButton=new Gtk::Button();

			PlateButton->set_label((*i)->GetName());
			PlateButton->signal_clicked().connect(sigc::bind<ApproachPlate *>(sigc::mem_fun(*this,&AirportInfoDialog::on_PlateButton_clicked),*i));
			ApproachPlatesVBox->add(*PlateButton);
		}

		ApproachPlatesScrolledWindow->add(*ApproachPlatesVBox);
		ApproachPlatesScrolledWindow->show_all();
	} else {
		ApproachPlatesFrame->hide();
	}

	if (AirportData->GetMETARs()->size()) {
		for (std::vector<METAR*>::iterator i=AirportData->GetMETARs()->begin(); i!=AirportData->GetMETARs()->end();i++) {
			if (i!=AirportData->GetMETARs()->begin()) {
				METARTextView->get_buffer()->insert_at_cursor("\n");
			}
			METARTextView->get_buffer()->insert_at_cursor((*i)->GetText());
		}
	} else {
		METARFrame->hide();
	}
}

AirportInfoDialog::~AirportInfoDialog()
{
	delete GtkDialog;
}

int AirportInfoDialog::Run() {
	if (!Data->HasActiveVolume()) {
		Gtk::MessageDialog *Message = new Gtk::MessageDialog("No active data volume",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		Message->run();
		delete Message;
		return 0;
	}

	return GtkDialog->run();
}

void AirportInfoDialog::on_AddtoRouteButton_clicked() {
	RouteData->Add(new RoutePoint(AirportData));
}

void AirportInfoDialog::on_CloseButton_clicked() {
	GtkDialog->response(Gtk::RESPONSE_NONE);
}

void AirportInfoDialog::on_PlateButton_clicked(ApproachPlate *PlateData) {
	PlateData->View(Data);
}
