/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#include <errno.h>
#include <string>
#include <sstream>
#include <cstring>
#ifdef WIN32
#include <process.h>
#endif
#include <glibmm/i18n.h>
#include <gtkmm/messagedialog.h>
#include <gtkmm/main.h>
#include "AirportSearchDialog.h"
#include "AirportInfoDialog.h"
#include "Airport.h"
#include "Util.h"

AirportSearchDialog::AirportSearchDialog(std::string *GladeFilename, DataStore *Data, Config *Cfg, Route *RouteData)
{
	try
	{
		refXml = Gnome::Glade::Xml::create(*GladeFilename, "AirportSearchDialog");
	}
	catch (const Gnome::Glade::XmlError& ex)
	{
		Gtk::MessageDialog *message = new Gtk::MessageDialog(_("Error reading data file: ")+*GladeFilename+"\n"+_("Program cannot continue."),
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		message->run();
		Gtk::Main::quit();
	}

	refXml->get_widget("AirportSearchDialog",GtkDialog);
	refXml->get_widget("AirportEntry",AirportEntry);
	refXml->get_widget("AirportsTreeView",AirportsTreeView);
	refXml->get_widget("InformationButton",InformationButton);
	refXml->get_widget("AddtoRouteButton",AddtoRouteButton);
	refXml->get_widget("CloseButton",CloseButton);
	refXml->get_widget("OnScreenKeypadParent",OnScreenKeypadParent);

	Keypad=new OnScreenKeypad();
	OnScreenKeypadParent->add(*Keypad);
	OnScreenKeypadParent->reorder_child(*Keypad,1);
	Keypad->show();
	Keypad->set_target(AirportEntry);

	AirportEntry->signal_changed().connect(sigc::mem_fun(this,&AirportSearchDialog::on_AirportEntry_changed));
	InformationButton->signal_clicked().connect(sigc::mem_fun(this,&AirportSearchDialog::on_InformationButton_clicked));
	AddtoRouteButton->signal_clicked().connect(sigc::mem_fun(this,&AirportSearchDialog::on_AddtoRouteButton_clicked));
	CloseButton->signal_clicked().connect(sigc::mem_fun(this,&AirportSearchDialog::on_CloseButton_clicked));

	this->GladeFilename=GladeFilename;
	this->Cfg=Cfg;
	this->Data=Data;
	this->RouteData=RouteData;

	MatchingItems.clear();

	ListStore = Gtk::ListStore::create(Columns);
	AirportsTreeView->set_model(ListStore);
	AirportsTreeView->append_column("ID", Columns.AirportIdent);
	AirportsTreeView->append_column("Name", Columns.AirportName);

	Gtk::TreeView::Column* pColumn = AirportsTreeView->get_column(0);
	if(pColumn)
		pColumn->set_sort_column(Columns.AirportIdent);

	pColumn = AirportsTreeView->get_column(1);
	if(pColumn)
		pColumn->set_sort_column(Columns.AirportName);

	if (!RouteData->GetFilename().length()) {
		AddtoRouteButton->set_sensitive(false);
	}
}

AirportSearchDialog::~AirportSearchDialog()
{
	delete GtkDialog;
}

int AirportSearchDialog::Run() {
	if (!Data->HasActiveVolume()) {
		Gtk::MessageDialog *Message = new Gtk::MessageDialog("No active data volume",
			false,Gtk::MESSAGE_ERROR,Gtk::BUTTONS_CANCEL,true);
		Message->run();
		delete Message;
		return 0;
	}

	return GtkDialog->run();
}

void AirportSearchDialog::on_AirportEntry_changed() {
	static std::string OldText,CurrText;
	std::vector<Airport*>::iterator Airporti;
	std::vector<MatchingItem>::iterator MatchingItemi;

	if (!Data->HasActiveVolume())
		return;	/* No data to search */

	CurrText=AirportEntry->get_text();

	if (CurrText.length()<=1 || (CurrText.length()<=2 && (toupper(CurrText[0])=='K'))) {
		MatchingItems.clear();
		OldText=CurrText;
	} else {
		if (CurrText.substr(0,CurrText.length()-1)==OldText && (MatchingItems.size()>0 && CurrText.length()>2)) {
			/* The new search is a subset of the old one */
			std::vector<MatchingItem> NewMatchingItems;
			for (MatchingItemi=MatchingItems.begin();MatchingItemi!=MatchingItems.end();MatchingItemi++) {
				if ((*MatchingItemi).Matchi(CurrText)) {
					NewMatchingItems.push_back(*MatchingItemi);
				}
			}
			MatchingItems=NewMatchingItems;
		} else {
			/* We have to search all of the airports */
			MatchingItems.clear();
			for (Airporti=Data->GetAirports()->begin();Airporti!=Data->GetAirports()->end();++Airporti) {
				if ((*Airporti)->Matchi(CurrText)) {
					MatchingItem NewItem(*Airporti);
					MatchingItems.push_back(NewItem);
					if (MatchingItems.size()>64) {
						/* We have too many matches - give up to avoid bogging down slower computers */
						MatchingItems.clear();
						ListStore->clear();
						OldText=CurrText;
						return;
					}
				}
			}
		}
	}

	ListStore->clear();

	for (MatchingItemi=MatchingItems.begin();MatchingItemi!=MatchingItems.end();MatchingItemi++) {
		Gtk::TreeModel::iterator iter = ListStore->append();
		Gtk::TreeModel::Row row = *iter;

		row[Columns.AirportIdent] = (*MatchingItemi).GetAirport()->GetID();
		row[Columns.AirportName] = (*MatchingItemi).GetAirport()->GetName();
		row[Columns.AirportData] = (*MatchingItemi).GetAirport();
	}

	if (MatchingItems.size()) {
		/* Select the first item by default */
		Glib::RefPtr<Gtk::TreeSelection> AirportSelection = AirportsTreeView->get_selection();
		Gtk::TreeModel::iterator iter = ListStore->children().begin();
		if(iter)
			AirportSelection->select(iter);
	}

	OldText=CurrText;
}

void AirportSearchDialog::on_InformationButton_clicked() {
	Glib::RefPtr<Gtk::TreeSelection> AirportSelection = AirportsTreeView->get_selection();
	Gtk::TreeModel::iterator iter = AirportSelection->get_selected();
	if (iter) {		/* If anything is selected */
		GtkDialog->hide();
		Gtk::TreeModel::Row row = *iter;
		AirportInfoDialog Dialog(GladeFilename,Data,Cfg,row[Columns.AirportData],RouteData);
		Dialog.Run();
	}
}

void AirportSearchDialog::on_AddtoRouteButton_clicked() {
	Glib::RefPtr<Gtk::TreeSelection> AirportSelection = AirportsTreeView->get_selection();
	Gtk::TreeModel::iterator iter = AirportSelection->get_selected();
	if (iter) {		/* If anything is selected */
		Gtk::TreeModel::Row row = *iter;
		RouteData->Add(new RoutePoint(row[Columns.AirportData]));
	}
}

void AirportSearchDialog::on_CloseButton_clicked() {
	GtkDialog->response(Gtk::RESPONSE_NONE);
}

AirportSearchDialogModelColumns::AirportSearchDialogModelColumns() {
	add(AirportIdent);
	add(AirportName);
	add(AirportData);
}

AirportSearchDialog::MatchingItem::MatchingItem(Airport *AirP) {
	this->AirP=AirP;
}

bool AirportSearchDialog::MatchingItem::Matchi(std::string str) {
	return AirP->Matchi(str);
}

Airport *AirportSearchDialog::MatchingItem::GetAirport() {
	return AirP;
}
