/*
 * Vifir - An electronic flight bag wannabe utility for aircraft pilots
 * Copyright (C) 2009 David Ludlow
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * dave@adsllc.com, PO Box 13070, Toledo, OH 43623
 */

#define __STDC_LIMIT_MACROS
#include <sstream>
#include <math.h>
#include <exception>
#include <cstring>
#include <cstdlib>

#include "Frequency.h"
#include "ARINC424_18.h"
#include "ImportXPlane900Data.h"

const std::string Frequency::UnitsText[4]= {
	"Hz",
	"KHz",
	"MHz",
	"GHz"
};

Frequency::Frequency(unsigned int RawFreq, unsigned int Precision, UnitsEnum Units, TypeEnum Type) {
	this->RawFreq=RawFreq;
	this->Precision=Precision;
	this->Units=Units;
	this->Type=Type;
}

double Frequency::ToDouble() {
	return RawFreq/pow(Precision,10.0);
}

std::string Frequency::FreqToString() {
	unsigned int p, factor, whole, decimal;
	std::ostringstream result;

	/* 12345,2 -> 123.45 */

	factor=1;
	for (p=0;p<Precision;p++) {
		factor*=10;
	}

	whole=RawFreq/factor;
	decimal=RawFreq%factor;

	result<<whole;
	if (Precision>0) {
		result<<"."<<decimal;
	}

	return result.str();
}

std::string Frequency::ToString() {
	return FreqToString()+" "+Frequency::UnitsText[Units];
}

/* Accessors */
unsigned int Frequency::GetRawFreq() {
	return RawFreq;
}

void Frequency::SetRawFreq(unsigned int RawFreq) {
	this->RawFreq=RawFreq;
}

unsigned char Frequency::GetPrecision() {
	return Precision;
}

void Frequency::SetPrecision(unsigned char Precision) {
	this->Precision=Precision;
}

enum Frequency::UnitsEnum Frequency::GetUnits() {
	return Units;
}

void Frequency::SetUnits(UnitsEnum Units) {
	this->Units=Units;
}

enum Frequency::TypeEnum Frequency::GetType() {
	return Type;
}

void Frequency::SetType(TypeEnum Type) {
	this->Type=Type;
}

std::string Frequency::GetDesc() {
	return Desc;
}

void Frequency::SetDesc(std::string Desc) {
	this->Desc=Desc;
}

std::string Frequency::GetAirportICAO()
{
	return AirportICAO;
}

bool Frequency::ParseNFD(std::string Line)
{
	struct arinc424_18_comm_struct *NFDData;
	char strbuf[sizeof(struct arinc424_18_comm_struct)];

	if (Line.length()!=sizeof(struct arinc424_18_comm_struct)+1)	/* Allow for LF */
	{
		return false;
	}

	if (strcmp(arinc424_18_parse_recordtype(Line.c_str()),ARINC424_18_RECTYPE_AIRPORT_COMMUNICATIONS)!=0)
	{
		return false;
	}

	NFDData=(struct arinc424_18_comm_struct *)Line.c_str();

	/* ICAO */
	memcpy(strbuf,NFDData->header.airportid,sizeof(NFDData->header.airportid));
	strbuf[sizeof(NFDData->header.airportid)]=0;
	AirportICAO=strbuf;

	/* Frequency */
	RawFreq=arinc424_18_parse_freq(NFDData->header.freq);
	Precision=3;
	Units=MHZ;
	Type=MISC;

	return true;
}

bool Frequency::ParseXPlane850(std::string *Line, std::string AirportICAO) {
	char *tok;
	char *buf;

	/* Copy to a normal C buffer for strtok_r() to work with */
	buf=(char*)malloc(Line->length()+1);
	if (!buf)
		return false;
	strncpy(buf,Line->c_str(),Line->length());
	buf[Line->length()]=0;

	try {
		/* Parse the fields */
		if (!(tok=strtok(buf," \t")))
			throw std::exception();
		switch (atoi(tok)) {
			case ImportXPlane900Data::ROWCODE_FREQ_WX:
				Type=ASOS;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_UNICOM:
				Type=UNICOM;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_CLD:
				Type=CLEARANCEDELIVERY;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_GROUND:
				Type=GROUND;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_TOWER:
				Type=TOWER;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_APP:
				Type=APPROACH;
				break;
			case ImportXPlane900Data::ROWCODE_FREQ_DEP:
				Type=DEPARTURE;
				break;
			default:
				Type=MISC;
				break;
		}
		if (!(tok=strtok(NULL," \t")))
			throw std::exception();
		float Freq=atof(tok);
		RawFreq=roundf(Freq*1000);
		if (RawFreq%100==20 || RawFreq%100==70)
			RawFreq+=5;	/* e.g. 123.22 -> 123.220 -> 123.225 */
		if (!(tok=strtok(NULL," \t")))
			throw std::exception();
		Desc=tok;
	}

	catch (...) {
		free(buf);
		return false;
	}

	Precision=3;
	Units=MHZ;
	this->AirportICAO=AirportICAO;

	free(buf);
	return true;
}

void Frequency::Clone(Frequency *SrcFreq)
{
	RawFreq=SrcFreq->GetRawFreq();
	Precision=SrcFreq->GetPrecision();
	Units=SrcFreq->GetUnits();
	Type=SrcFreq->GetType();
	Desc=SrcFreq->GetDesc();
	AirportICAO=SrcFreq->GetAirportICAO();
}

bool Frequency::MakeBinary(struct Frequency_Binary_struct *BinaryData) {
	/* Sanity checking */
	if (
		RawFreq>UINT32_MAX || RawFreq<0 ||
		Units>UINT8_MAX || Units<0 ||
		Type>UINT8_MAX || Type<0 ||
		Desc.size()>sizeof(BinaryData->Desc)
		) {
		return false;
	}

	BinaryData->RawFreq=RawFreq;
	BinaryData->Precision=Precision;
	BinaryData->Units=Units;
	BinaryData->Type=Units;
	strncpy(BinaryData->Desc,Desc.c_str(),sizeof(BinaryData->Desc));

	return true;
}

void Frequency::SetFromBinary(struct Frequency_Binary_struct *BinaryData) {
	RawFreq=BinaryData->RawFreq;
	Precision=BinaryData->Precision;
	Units=(Frequency::UnitsEnum)BinaryData->Units;
	Type=(Frequency::TypeEnum)BinaryData->Type;
	Desc.assign(BinaryData->Desc,sizeof(BinaryData->Desc));
	/* Reset the size to the actual number of characters */
	Desc=Desc.c_str();
}

